/* 
 * File:   ConfigComboPopup.h
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : LGPL
 * Copyright Saleem EDAH-TALLY - © 2017
 *
 * Created on 3 mars 2017, 19:14
 */

#ifndef CONFIGCOMBOPOPUP_H
#define CONFIGCOMBOPOPUP_H

#include <wx/wx.h>
#include <wx/config.h>
#include "PopupTransientWindow.h"

class TransientPopupComboEvent;

/**
 * Creates a transient popup with the intent of adding and deleting
 * an item of an editable combo box. A description for each item can be saved.
 * The items are typically stored in a wxConfig file. 
 * This class does not read or write to storhe. It expects an event
 * handler provided by an application to do that.
 * @param evh
 */
class ConfigComboPopup
{
public:
    friend class TransientPopupComboEvent;

    ConfigComboPopup(TransientPopupComboEvent * evh);
    virtual ~ConfigComboPopup();

    void ShowTransientPopUp(wxControl * owner);

    PopupTransientWindow * GetPopupTransientWindow()
    {
        return m_popup;
    }

    wxPanel * GetPanel()
    {
        return m_pan;
    }

    wxSizer * GetSizerMain()
    {
        return m_sz;
    }

    wxSizer * GetSizerButton()
    {
        return m_szBtn;
    }

    wxTextCtrl * GetTxtInfo()
    {
        return m_txtInfo;
    }

    wxButton * GetBtnSave()
    {
        return m_btnSave;
    }

    wxButton * GetBtnDelete()
    {
        return m_btnDelete;
    }

private:
    PopupTransientWindow * m_popup;
    wxPanel * m_pan;
    wxBoxSizer * m_sz;
    wxTextCtrl * m_txtInfo;
    wxBoxSizer * m_szBtn;
    wxButton * m_btnSave;
    wxButton * m_btnDelete;
    TransientPopupComboEvent * m_evh;

    void TransientButtonSave(wxCommandEvent& evt);
    void TransientButtonDelete(wxCommandEvent& evt);
    void TransientPopupDestroy(wxWindowDestroyEvent& evt);
};

/**
 * Pure virtual callbacks that an application must implement to manage combo
 * items.
 */
class TransientPopupComboEvent
{
public:

    TransientPopupComboEvent()
    {
    };

    virtual ~TransientPopupComboEvent()
    {
    };

    virtual void OnShow() = 0;
    virtual void OnSave() = 0;
    virtual void OnDelete() = 0;
};

#endif /* CONFIGCOMBOPOPUP_H */

