/* 
 * File:   CopyFiles.h
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : LGPL
 * Copyright Saleem EDAH-TALLY - © 2014
 *
 * Created on 28 septembre 2014, 17:53
 */

#ifndef COPYFILES_H
#define COPYFILES_H

#include <wx/wx.h>
#include <wx/thread.h>
#include <wx/progdlg.h>

class FileCopyEvent;
class FileCopyProgress;

/**
 * Does the job of copying or moving files from src to dest asynchronously.
 * Show a progress dialog at a provided location.
 * Forwards messages to the caller via an event handler.
 * @param parent
 * @param evh
 * @param src
 * @param dest
 * @param progDialogPosition
 * @param move
 * @param buffSize
 */
class FileCopy : public wxThread
{
public:
    FileCopy(wxWindow * parent, FileCopyEvent * evh,
             const wxString& src, const wxString& dest,
             const wxPoint progDialogPosition = wxDefaultPosition,
             bool move = false, const int buffSize = (1024 * 1024));
    virtual ~FileCopy();

    virtual ExitCode Entry();

private:
    wxString m_src, m_dest;
    bool m_move;
    unsigned int m_bufSize;
    wxWindow * m_owner;
    wxWeakRef<FileCopyEvent> m_evh;
    wxProgressDialog * m_pdlg;
    FileCopyProgress * m_fcp;
};

/**
 * Callbacks to applications. Message transmitter.
 */
class FileCopyEvent : public wxTrackable
{
public:
    FileCopyEvent();
    virtual ~FileCopyEvent();

    virtual void OnRun(wxThreadIdType id, wxProgressDialog * pdlg)
    {
    };

    virtual void OnDeath(wxThreadIdType id)
    {
    };

    virtual void OnError(const wxString& src, const wxString& dest)
    {
    };

    virtual void OnCancel(const wxString& src, const wxString& dest)
    {
    };

    virtual void OnSuccess(const wxString& src, const wxString& dest)
    {
    };

    virtual void OnAbortExisting(const wxString& src, const wxString& dest)
    {
    };
};

/**
 * Updates a progress dialog according to the extent of a file copy process.
 * @param pdlg
 * @param fileSize
 */
class FileCopyProgress : public wxEvtHandler
{
public:
    FileCopyProgress(wxProgressDialog * pdlg, const unsigned long fileSize);
    virtual ~FileCopyProgress();
    void SetParams(unsigned long done);
    void DoUpdate();
    
private:
    wxWeakRef<wxProgressDialog> m_pdlg;
    unsigned long m_done;
    unsigned long m_fileSize;
};

/**
 * Defines an array of rectangular locations to position multiple progress
 * dialogs. Starts in the top right screen position, then downward and leftward.
 * If there are too many dialogs, overlaps according to the same scheme.
 */
class ProgressDialogTiles : public wxTrackable
{
public:
    ProgressDialogTiles();
    ~ProgressDialogTiles();

    /**
     * Provides the location of a progress dialog to be positioned.
     * @return 
     */
    const wxRect GetCurrentTile();
private:
    wxSize m_defSize;
    unsigned int m_hMax, m_vMax;
    wxVector<wxRect> m_tiles;
    unsigned int m_current;
};
#endif /* COPYFILES_H */

