/* 
 * File:   DBusCB7Workers.h
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : GPL
 * Copyright Saleem EDAH-TALLY - © 2017
 *
 * Created on 5 mars 2017, 15:18
 */

#ifndef DBUSCB7WORKERS_H
#define DBUSCB7WORKERS_H

#include "dbus_cb7_adaptor.hpp"
#include "dbus_cb7_proxy.hpp"
#include <wx/wx.h>
#include <wx/thread.h>
#include "XCB7.h"

class XCB7;

/**
 * A DBus adaptor for the application.
 * Contains methods (and signals) callable via DBus.
 * Only one (discardable) method is implemented here.
 * An adaptor is required so that the application can be noticed by the remote
 * XSane proxy when scan is over.
 */
class DBusCB7Adaptor : public Application::CB7_adaptor,
public DBus::IntrospectableAdaptor,
public DBus::ObjectAdaptor
{
public:
    DBusCB7Adaptor(DBus::Connection &connection, XCB7 * owner);
    virtual ~DBusCB7Adaptor();

    virtual void Show(const bool& how);
private:
    XCB7 * m_owner;

};

/**
 * Starts the adaptor which is a blocking process, so we need a
 * background thread.
 * @param connection
 * @param owner
 */
class DBusCB7AdaptorWorker : public wxThread
{
public:
    DBusCB7AdaptorWorker(DBus::Connection &connection, XCB7 * owner);
    virtual ~DBusCB7AdaptorWorker();

    virtual ExitCode Entry();
};

/**
 * Unused. No need to talk to other CB7 instances.
 * @param connection
 * @param service
 */
class DBusCB7Proxy : public Application::CB7_proxy,
public DBus::IntrospectableProxy,
public DBus::ObjectProxy
{
public:
    DBusCB7Proxy(DBus::Connection &connection, const char * service); //Unused
    virtual ~DBusCB7Proxy();
};

#endif /* DBUSCB7WORKERS_H */

