/* 
 * File:   FileWorker.h
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : GPL
 * Copyright Saleem EDAH-TALLY - © 2017
 *
 * Created on 5 mars 2017, 20:56
 */

#ifndef FILEWORKER_H
#define FILEWORKER_H

#include "CopyFiles.h"
#include <wx/wx.h>
#include <wx/dnd.h>
#include "XCB7.h"

class XCB7;
class FileCopyEVH;
class DNDFileDataTransfer;

/**
 * Appendix to XCB7. Manages file copy operations.
 * @param owner
 */
class FileWorker
{
public:
    friend class FileCopyEVH;
    friend class DNDFileDataTransfer;

    FileWorker(XCB7 * owner);
    virtual ~FileWorker();
private:
    XCB7 * m_owner;
    wxSizer * m_szMain;
    wxCollapsiblePane * m_paneFiles;
    wxCheckBox * m_cbMove;
    wxCheckBox * m_cbTimeStamp;
    wxCheckBox * m_cbAppendName;
    wxCheckBox * m_cbPreferOriginal;
    wxTextCtrl * m_txtResults;
    wxButton * m_btnSelectFile;
    wxConfig * m_config;
    /**
     * Array of id of started threads
     */
    wxArrayLong m_threads;
    ProgressDialogTiles * m_tiles;

    /**
     * For each item of filenames, creates a destination file name as per options,
     * starts a thread that does the job.
     * @param filenames
     * @return 
     */
    bool DoRenameAndCopy(const wxArrayString& filenames);

    /**
     * Shows or hides a control showing results of file copy operations.
     * @param evt
     */
    void OnPaneFilesChanged(wxCollapsiblePaneEvent& evt);

    /**
     * Remember user choices. Manages consistency between options.
     * @param evt
     */
    void OnCbMove(wxCommandEvent& evt);
    void OnCbTimeStamp(wxCommandEvent& evt);
    void OnCbAppendName(wxCommandEvent& evt);
    void OnCbPreferOriginal(wxCommandEvent& evt);

    /**
     * Select files via a dialog rather than drag and drop.
     * @param evt
     */
    void OnBtnSelectFile(wxCommandEvent& evt);
};

/**
 * Implementation of callbacks from FileCopy. Mainly to display results
 * of file copy.
 * @param caller
 * @param owner
 */
class FileCopyEVH : public FileCopyEvent
{
public:
    FileCopyEVH(FileWorker * caller, wxWindow * owner);
    virtual ~FileCopyEVH();

    virtual void OnRun(wxThreadIdType id, wxProgressDialog * pdlg);
    virtual void OnDeath(wxThreadIdType id);
    virtual void OnError(const wxString& src, const wxString& dest);
    virtual void OnCancel(const wxString& src, const wxString& dest);
    virtual void OnSuccess(const wxString& src, const wxString& dest);
    virtual void OnAbortExisting(const wxString& src, const wxString& dest);

private:
    FileWorker * m_caller;
    wxWindow * m_owner;
};

/**
 * Drop target for file copy using drag and drop.
 * @param parent
 * @param source
 */
class DNDFileDataTransfer : public wxFileDropTarget
{
public:
    DNDFileDataTransfer(FileWorker * parent, wxWindow * source);
    virtual ~DNDFileDataTransfer();

    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);

private:
    wxWindow * m_pane;
    FileWorker * m_owner;
};

#endif /* FILEWORKER_H */

