/*
* File:   InsaneWorker.h
* Author: Saleem Edah-Tally - nmset@yandex.com
* License : CeCILL-C
* Copyright Saleem Edah-Tally - © 2023
*
* Created on 14 02 2024, 13:00
*/

#ifndef INSANEWORKER_H
#define INSANEWORKER_H

#include <libinsane/capi.h>
#include <string>

class InsaneWorkerEvent;

class InsaneWorker
{
public:
    enum OUTPUT_FILETYPE {PNG, PDF};
    
    InsaneWorker(const std::string& deviceId,
                 const std::string& filePath, OUTPUT_FILETYPE fileType = PNG,
                 const std::string& source = "FlatBed",
                 const std::string& mode = "Color", int resolution = 300,
                 InsaneWorkerEvent* evh = nullptr, lis_api* api = nullptr);
    ~InsaneWorker();

    bool Init();
    lis_api * GetInsaneApi() const
    {
        return m_api;
    }
private:
    lis_api * m_api = nullptr;
    lis_item * m_item = nullptr;
    InsaneWorkerEvent * m_evh = nullptr;
    
    std::string m_deviceId;
    std::string m_fileName;
    OUTPUT_FILETYPE m_fileType = PNG;
    std::string m_source = "FlatBed";
    std::string m_mode = "Color";
    int m_resolution = 300;
};

class InsaneWorkerEvent
{
public:
    InsaneWorkerEvent();
    ~InsaneWorkerEvent();
    
    virtual void OnInsaneError(const std::string& message);
};

#endif // INSANEWORKER_H
