/* 
 * File:   MiscTools.cpp
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : LGPL
 * Copyright Saleem EDAH-TALLY - © 2017
 * 
 * Created on 4 mars 2017, 11:40
 */

#include "MiscTools.h"
#include "globals.h"
#include <uuid/uuid.h>
#include <wx/notifmsg.h>
#include <wx/msgdlg.h>

MiscTools::MiscTools()
{
}

MiscTools::~MiscTools()
{
}

const wxString MiscTools::MakeUUID()
{
    uuid_t uuid;
    char c_uuid[36];
    uuid_generate(uuid);
    uuid_unparse(uuid, c_uuid);
    return wxString(c_uuid);
}

void MiscTools::FillConfigItemCombo(wxComboBox* cmb, wxConfig* config, const wxString path)
{
    const wxString oldPath = config->GetPath();
    config->SetPath(path);
    wxString key;
    long idx = 0;
    cmb->Clear();
    while (config->GetNextEntry(key, idx))
    {
        if (!key.IsEmpty())
        {
            cmb->Append(config->Read(key));
        }
    }
    config->SetPath(oldPath);
}

bool MiscTools::ConfigPathHasValue(wxConfig* thisConfig, const wxString path, const wxString value)
{
    const wxString oldPath = thisConfig->GetPath();
    thisConfig->SetPath(path);
    wxString key;
    long idx = 0;
    while (thisConfig->GetNextEntry(key, idx))
    {
        if (!key.IsEmpty())
        {
            if (thisConfig->Read(key) == value)
            {
                thisConfig->SetPath(oldPath);
                return true;
            }
        }
    }
    thisConfig->SetPath(oldPath);
    return false;
}

void MiscTools::ConfigRemoveByValue(wxConfig* thisConfig, const wxString path, const wxString value)
{
    const wxString oldPath = thisConfig->GetPath();
    thisConfig->SetPath(path);
    wxString key;
    long idx = 0;
    while (thisConfig->GetNextEntry(key, idx))
    {
        if (!key.IsEmpty())
        {
            if (thisConfig->Read(key) == value)
            {
                thisConfig->DeleteEntry(key);
                break;
            }
        }
    }
    thisConfig->SetPath(oldPath);
}

void MiscTools::SaveSizePos(wxConfig* config, wxWindow* wind, const wxString& pathPrefix)
{
    config->Write(pathPrefix + _T("/Width"), wind->GetSize().GetWidth());
    config->Write(pathPrefix + _T("/Height"), wind->GetSize().GetHeight());
    config->Write(pathPrefix + _T("/X"), wind->GetPosition().x);
    config->Write(pathPrefix + _T("/Y"), wind->GetPosition().y);
    config->Flush();
}

void MiscTools::RestoreSizePos(wxConfig* config, wxWindow* wind, const wxString& pathPrefix)
{
    long w = 0, h = 0;
    config->Read(pathPrefix + _T("/Width"), &w);
    config->Read(pathPrefix + _T("/Height"), &h);
    if (w && h)
    {
        wind->SetSize(w, h);
    }
    bool res = false;
    long x = 0, y = 0;
    res = config->Read(pathPrefix + _T("/X"), &x);
    res = config->Read(pathPrefix + _T("/Y"), &y);
    if (!res)
    {
        wind->Centre();
        return;
    }
    wind->SetPosition(wxPoint(x, y));
}

void MiscTools::ShowTransientPopup(wxPopupTransientWindow* p, wxWindow* content,
                                   const int width, const int height)
{
    wxBoxSizer * sz = new wxBoxSizer(wxVERTICAL);
    p->SetSizer(sz);
    p->SetSize(width, height);
    sz->Add(content, 0, wxGROW | wxGROW | wxALL);
    sz->Layout();
    wxScreenDC dc;
    const int screenWidth = dc.GetSize().GetWidth();
    const int screenHeight = dc.GetSize().GetHeight();
    const wxPoint mousePos = wxGetMousePosition();
    wxPoint dest(mousePos.x, mousePos.y);
    if (mousePos.x > (screenWidth - width)) dest.x = (screenWidth - width);
    if (mousePos.y > (screenHeight - height)) dest.y = (screenHeight - height);
    if (width > screenWidth) dest.x = 0;
    if (height > screenHeight) dest.y = 0;
    p->SetPosition(dest);
    p->Popup();
}

void MiscTools::MessageBox(const wxString& msg, const bool notify)
{
    wxApp * app = static_cast<wxApp*> (wxApp::GetInstance());
    if (!notify)
    {
        wxMessageBox(msg, _APPNAME_, wxOK, app->GetTopWindow());
    }
    else
    {
        wxNotificationMessage notifMsg(_APPNAME_, msg, app->GetTopWindow());
        notifMsg.Show();
    }
}

wxTextValidator* MiscTools::MakeFileNameValidator(bool excludeSpace)
{
    wxTextValidator * tval = new wxTextValidator(wxFILTER_EXCLUDE_CHAR_LIST);
    const wxString xcl = wxFileName::GetForbiddenChars();
    wxArrayString arr;
    for (unsigned short i = 0; i < xcl.Length(); i++)
    {
        arr.Add(xcl.GetChar(i));
    }
    if (excludeSpace) arr.Add(_T(" "));
    arr.Add(wxFileName::GetPathSeparator());
    tval->SetExcludes(arr);
    return tval;
}

wxString MiscTools::GetRealBaseName(const wxFileName& fn)
{
    wxString basename = fn.GetFullName(); // Not GetName()
    do
    {
        basename = wxFileName(basename).GetName();
    }
    while (wxFileName(basename).HasExt());
    return basename;
}

wxString MiscTools::GetFullExt(const wxFileName& fn)
{
    wxString basename = fn.GetFullName(); // Not GetName()
    wxString ext = wxEmptyString;
    do
    {
        ext = wxFileName(basename).GetExt() + _DOT_ + ext;
        basename = wxFileName(basename).GetName();
    }
    while (wxFileName(basename).HasExt());
    return ext.Left(ext.Len() - 1);
}
