/* 
 * File:   MiscTools.h
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : LGPL
 * Copyright Saleem EDAH-TALLY - © 2017
 *
 * Created on 4 mars 2017, 11:40
 */

#ifndef MISCTOOLS_H
#define MISCTOOLS_H

#include <wx/wx.h>
#include <wx/config.h>
#include <wx/popupwin.h>

/**
 * Miscellaneous static functions.
 */
class MiscTools
{
public:
    MiscTools();
    virtual ~MiscTools();

    /**
     * Generates a UUID of default variant.
     */
    static const wxString MakeUUID();

    /**
     * Fills a wxComboBox with items stored in a wxConfig file.
     * @param cmb
     * @param config
     * @param path
     */
    static void FillConfigItemCombo(wxComboBox * cmb, wxConfig * config, const wxString path);
    /**
     * Finds if any key in a known wxConfig path has a value.
     * @param thisConfig
     * @param path
     * @param value
     * @return 
     */
    static bool ConfigPathHasValue(wxConfig * thisConfig, const wxString path, const wxString value);

    /**
     * Deletes a key with a known value in a known wxConfig path.
     * @param thisConfig
     * @param path
     * @param value
     */
    static void ConfigRemoveByValue(wxConfig * thisConfig, const wxString path, const wxString value);

    /**
     * Saves geometry and position of a wxWindow.
     * @param config
     * @param wind
     * @param pathPrefix
     */
    static void SaveSizePos(wxConfig * config, wxWindow * wind, const wxString& pathPrefix);

    /**
     * Restores geometry and position of a wxWindow.
     * @param config
     * @param wind
     * @param pathPrefix
     */
    static void RestoreSizePos(wxConfig * config, wxWindow * wind, const wxString& pathPrefix);

    /**
     * Shows a transient popup at mouse cursor.
     * Manages positioning the popup relative to screen edges and popup dimensions.
     * @param p
     * @param content
     * @param width
     * @param height
     */
    static void ShowTransientPopup(wxPopupTransientWindow * p, wxWindow* content, const int width, const int height);

    /**
     * Shows a message as a modal dialog or as a notification.
     * @param msg
     * @param notify
     */
    static void MessageBox(const wxString& msg, const bool notify = false);

    /**
     * Creates a validator excluding file name forbidden characters, path
     * separator and optionally space character.
     * @param excludeSpace
     * @return 
     */
    static wxTextValidator* MakeFileNameValidator(bool excludeSpace = true);

    /**
     * Gets the basename of a file with multiple extensions (ex foo.pkg.tar.xz).
     * @param fn
     * @return 
     */
    static wxString GetRealBaseName(const wxFileName& fn);

    /**
     * Gets all extensions of a file (ex foo.pkg.tar.xz).
     * @param fn
     * @return 
     */
    static wxString GetFullExt(const wxFileName& fn);

private:

};

#endif /* MISCTOOLS_H */

