/* 
 * File:   PdfBinder.cpp
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * All rights reserved.
 * Copyright Saleem EDAH-TALLY - © 2005
 * 
 * Created on October 22, 2019, 3:43 PM
 */

#include "PdfBinder.h"
#include <qpdf/QPDFPageDocumentHelper.hh>
#include <qpdf/QPDFWriter.hh>
#include <qpdf/QPDFExc.hh>
#include <vector>

using namespace std;

PdfBinder::PdfBinder() {
    m_what = wxEmptyString;
}

PdfBinder::~PdfBinder() {
}
bool PdfBinder::Merge(const wxArrayString& files,
                      const wxString& outFile)
{
    // Almost all QPDF functions seen so far return void !!!
    m_what.Clear();
    try
    {
        QPDF outpdf;
        outpdf.emptyPDF();
        QPDFPageDocumentHelper qpdh(outpdf);
        for (uint i = 0; i < files.GetCount(); i++)
        {
            QPDF inpdf;
            inpdf.processFile(files.Item(i).utf8_str().data());
            vector<QPDFPageObjectHelper> pages = QPDFPageDocumentHelper(inpdf).getAllPages();
            vector<QPDFPageObjectHelper>::iterator it;
            for (it = pages.begin(); it != pages.end(); it++)
            {
                QPDFPageObjectHelper page(*it);
                qpdh.addPage(page, false);
            }
        }
        QPDFWriter outpdfw(outpdf, outFile.utf8_str().data());
        outpdfw.write();
    }
    catch (QPDFExc ex)
    { // Never called here
        m_what = wxString(ex.what());
    }
    catch (std::logic_error ex)
    { // Internal errors
        m_what = wxString(ex.what());
    }
    catch (std::runtime_error ex)
    {
        m_what = wxString(ex.what());
    }
    return (m_what.IsEmpty());
}

