/* 
 * File:   XCB7.cpp
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : GPL
 * Copyright Saleem EDAH-TALLY - © 2017
 * 
 * Created on 27 février 2017, 22:08
 */

#include "gcc_version.h"
#include "XCB7.h"
#include "globals.h"
#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/cmdline.h>
#include "MiscTools.h"
#include "ConfigEditorPopup.h"
#include "TimeredStatusBar.h"
#include "page.xpm"

#include <iostream>
using namespace std;

XCB7::XCB7(wxWindow* parent, wxWindowID id, const wxString& caption,
           const wxPoint& pos, const wxSize& size, long style)
: CB7(parent, id, caption, pos, size, style)
{
    m_ConfigPopup = NULL;
    m_config = NULL;
    m_configItems = NULL;
    m_configTag = wxEmptyString;
    m_xsaneWorker = NULL;
    m_fileWorker = NULL;
}

XCB7::~XCB7()
{
    if (m_config)
        MiscTools::SaveSizePos(m_config, this, _SLASH_ + _APPNAME_);
    delete m_fileWorker;
    delete m_xsaneWorker;
    delete m_ConfigPopup;
    delete m_configItems;
    delete m_config;
}

bool XCB7::ParseCmdLine()
{
    wxCmdLineParser p;
    p.SetCmdLine(wxApp::GetInstance()->argc, wxApp::GetInstance()->argv);
    p.SetSwitchChars(_T("-"));
    p.AddOption(_T("c"), wxEmptyString, _("Config file tag."));
    p.AddSwitch(_T("v"), wxEmptyString, _("Show version and quit."));
    p.AddSwitch(_T("h"), wxEmptyString, _("Show help and quit."));
    p.Parse(false);
    if (p.Found(_T("c")))
    {
        p.Found(_T("c"), &m_configTag);
        return true;
    }
    if (p.Found(_T("h")))
    {
        p.Usage();
        return false; //Exit code is 255, not clean.
    }
    if (p.Found(_T("v")))
    {
        cout << (_APPNAME_ + _(" - version ") + _APPVERSION_) << endl;
        return false;
    }
    return true;
}

void XCB7::Start()
{
    SetTitle(_APPTITLE_ + _T(" - ") + _APPVERSION_);
    const wxString configDir = wxFileConfig::GetLocalFile(_APPNAME_, wxCONFIG_USE_SUBDIR).GetPath();
    if (!wxFileName::Exists(configDir))
        wxFileName::Mkdir(configDir);
    const wxString configBaseName = m_configTag.IsEmpty()
            ? _APPNAME_
            : _APPNAME_ + _DASH_ + m_configTag;
    m_config = new wxFileConfig(_APPNAME_, _T("NMSET"), configBaseName,
                                wxEmptyString, wxCONFIG_USE_SUBDIR);
    m_configItems = new wxFileConfig(_APPNAME_, _T("NMSET"), configBaseName + _T("-items"),
                                     wxEmptyString, wxCONFIG_USE_SUBDIR);
    spnYear->SetRange(2007, wxDateTime::Now().GetYear());
    spnYear->SetValue(m_config->Read(_T("/Last/Year"), wxDateTime::Now().GetYear()));
    spnYear->Bind(wxEVT_SPINCTRL, &XCB7::OnSpnYearChange, this);

    MiscTools::FillConfigItemCombo(cmbFolder, m_configItems, _T("/Folder"));
    cmbFolder->Bind(wxEVT_LEFT_DCLICK, &XCB7::OnCmbFolderDClick, this);
    cmbFolder->Bind(wxEVT_COMMAND_COMBOBOX_SELECTED, &XCB7::OnCmbFolderSelect, this);
    cmbFolder->SetValidator(*MiscTools::MakeFileNameValidator());

    MiscTools::FillConfigItemCombo(cmbLabel, m_configItems, _T("/Label"));
    cmbLabel->Bind(wxEVT_LEFT_DCLICK, &XCB7::OnCmbLabelDClick, this);
    cmbLabel->Bind(wxEVT_COMMAND_COMBOBOX_SELECTED, &XCB7::OnCmbLabelSelect, this);
    cmbLabel->SetValidator(*MiscTools::MakeFileNameValidator());

    MiscTools::FillConfigItemCombo(cmbHint, m_configItems, _T("/Hint"));
    cmbHint->Bind(wxEVT_LEFT_DCLICK, &XCB7::OnCmbHintDClick, this);
    cmbHint->Bind(wxEVT_COMMAND_COMBOBOX_SELECTED, &XCB7::OnCmbHintSelect, this);
    cmbHint->SetValidator(*MiscTools::MakeFileNameValidator());

    MiscTools::FillConfigItemCombo(cmbFormat, m_configItems, _T("/Format"));
    cmbFormat->Bind(wxEVT_LEFT_DCLICK, &XCB7::OnCmbFormatDClick, this);
    cmbFormat->Bind(wxEVT_COMMAND_COMBOBOX_SELECTED, &XCB7::OnCmbFormatSelect, this);
    cmbFormat->Select(cmbFormat->FindString(m_config->Read(_T("/Last/Format"))));
    cmbFormat->SetValidator(*MiscTools::MakeFileNameValidator());

    dpkRepository->GetTextCtrl()->SetEditable(false);
    dpkRepository->SetPath(m_config->Read(_T("DocRoot"), wxGetUserHome()));
    dpkRepository->Bind(wxEVT_DIRPICKER_CHANGED, &XCB7::OnDpkRepositoryChange, this);
    dpkRepository->GetTextCtrl()->Bind(wxEVT_LEFT_DCLICK, &XCB7::OnDpkDoubleClick, this);

    bool res = false;
    if (!m_config->HasEntry(_T("/Scanner/ADF/Automatic")))
        res = m_config->Write(_T("/Scanner/ADF/Automatic"), false);
    if (!m_config->HasEntry(_T("/Scanner/ADF/DoubleSided")))
        res = m_config->Write(_T("/Scanner/ADF/DoubleSided"), false);
    if (!m_config->HasEntry(_T("/Scanner/ADF/Total")))
        res = m_config->Write(_T("/Scanner/ADF/Total"), 1);
    if (!m_config->HasEntry(_T("/Scanner/Bind")))
        res = m_config->Write(_T("/Scanner/Bind"), true);
    if (res)
        m_config->Flush();
    lblNewDoc->Bind(wxEVT_RIGHT_UP, &XCB7::OnLblNewDocRightClick, this);
    lblNewDoc->Bind(wxEVT_LEFT_UP, &XCB7::OnLblNewDocLeftClick, this);

    txtNewDoc->Bind(wxEVT_KEY_UP, &XCB7::OnTxtNewDocKeyPressed, this);
    txtResults->Bind(wxEVT_KEY_UP, &XCB7::OnTxtResultsKeyPressed, this);

    btnAbout->Bind(wxEVT_COMMAND_BUTTON_CLICKED, &XCB7::OnBtnAbout, this);

    Bind(wxEVT_CLOSE_WINDOW, &XCB7::OnCloseWindow, this);

    TimeredStatusBar * sb = new TimeredStatusBar(this);
    SetStatusBar(sb);
    m_xsaneWorker = new XSaneWorker(this, sb);
    m_fileWorker = new FileWorker(this);
    
    m_xsaneWorker->PopulateScannerList();
    btnRefresh->Bind(wxEVT_COMMAND_BUTTON_CLICKED, &XCB7::OnBtnRefreshScanners, this);

    MiscTools::RestoreSizePos(m_config, this, _SLASH_ + _APPNAME_);
    SetIcon(wxICON(page)); //https://www.iconfinder.com/icons/473800/clipboard_document_file_notepad_plan_schedule_write_icon#size=128
    wxImage(page_xpm).SaveFile(configDir + wxFileName::GetPathSeparator()
                               + _T("page.xpm"), wxBITMAP_TYPE_XPM);

}

void XCB7::OnCmbFolderDClick(wxMouseEvent& evt)
{
    wxDELETE(m_ConfigPopup);
    if (cmbFolder->GetValue().IsEmpty())
    {
        evt.Skip();
        return;
    }
    m_ConfigPopup = new ConfigComboPopup(
                                         new TransientPopupEVH(this, cmbFolder, _T("Folder"), _T("Folder-TTT"))
                                         );
    m_ConfigPopup->ShowTransientPopUp(cmbFolder);
    evt.Skip();
}

void XCB7::OnCmbFolderSelect(wxCommandEvent& evt)
{
    cmbFolder->SetToolTip(m_configItems->Read(_T("/Folder-TTT/" + cmbFolder->GetValue())));
    evt.Skip();
}

void XCB7::OnCmbLabelDClick(wxMouseEvent& evt)
{
    wxDELETE(m_ConfigPopup);
    if (cmbLabel->GetValue().IsEmpty())
    {
        evt.Skip();
        return;
    }
    m_ConfigPopup = new ConfigComboPopup(
                                         new TransientPopupEVH(this, cmbLabel, _T("Label"), _T("Label-TTT"))
                                         );
    m_ConfigPopup->ShowTransientPopUp(cmbLabel);
    evt.Skip();
}

void XCB7::OnCmbLabelSelect(wxCommandEvent& evt)
{
    cmbLabel->SetToolTip(m_configItems->Read(_T("/Label-TTT/" + cmbLabel->GetValue())));
    evt.Skip();
}

void XCB7::OnCmbHintDClick(wxMouseEvent& evt)
{
    wxDELETE(m_ConfigPopup);
    if (cmbHint->GetValue().IsEmpty())
    {
        evt.Skip();
        return;
    }
    m_ConfigPopup = new ConfigComboPopup(
                                         new TransientPopupEVH(this, cmbHint, _T("Hint"), _T("Hint-TTT"))
                                         );
    m_ConfigPopup->ShowTransientPopUp(cmbHint);
    evt.Skip();
}

void XCB7::OnCmbHintSelect(wxCommandEvent& evt)
{
    cmbHint->SetToolTip(m_configItems->Read(_T("/Hint-TTT/" + cmbHint->GetValue())));
    evt.Skip();
}

void XCB7::OnCmbFormatDClick(wxMouseEvent& evt)
{
    wxDELETE(m_ConfigPopup);
    if (cmbFormat->GetValue().IsEmpty())
    {
        evt.Skip();
        return;
    }
    m_ConfigPopup = new ConfigComboPopup(
                                         new TransientPopupEVH(this, cmbFormat, _T("Format"), _T("Format-TTT"))
                                         );
    m_ConfigPopup->ShowTransientPopUp(cmbFormat);
    evt.Skip();
}

void XCB7::OnCmbFormatSelect(wxCommandEvent& evt)
{
    cmbFormat->SetToolTip(m_configItems->Read(_T("/Format-TTT/" + cmbFormat->GetValue())));
    m_config->Write(_T("/Last/Format"), cmbFormat->GetValue());
    evt.Skip();
}

void XCB7::OnSpnYearChange(wxSpinEvent& evt)
{
    m_config->Write(_T("/Last/Year"), spnYear->GetValue());
    m_config->Flush();
    evt.Skip();
}

void XCB7::OnDpkRepositoryChange(wxFileDirPickerEvent& evt)
{
    m_config->Write(_T("/DocRoot"), dpkRepository->GetPath());
    m_config->Flush();
    evt.Skip();
}

void XCB7::OnLblNewDocRightClick(wxMouseEvent& evt)
{
    ConfigEditorPopup * cep = new ConfigEditorPopup(this, m_config);
    PopupTransientWindow * ptw = cep->CreatePopup();
    if (!ptw)
    {
        evt.Skip();
        return;
    }
    wxCheckBox * cb = cep->AddCheckBox(_("ADF :"), _T("/Scanner/ADF/Automatic"));
    cb->SetToolTip(_("Use an Automatic Document Feeder."));
    cb = cep->AddCheckBox(_("Double sided :"), _T("/Scanner/ADF/DoubleSided"));
    cb->SetToolTip(_("Ignored if no Automatic Document Feeder declared."));
    wxCheckBox * cbBind = cep->AddCheckBox(_("Bind :"), _T("/Scanner/Bind"));
    cbBind->SetToolTip(_("Concerns PDF documents only. Bind in a single file."));
    wxSpinCtrl * spn = cep->AddSpinCtrl(_("Total :"), _T("/Scanner/ADF/Total"));
    spn->SetRange(1, 50);
    spn->SetToolTip(_("Total number of scan to do (not total number of pages)."));
    cep->ShowPopup();
    evt.Skip();
}

void XCB7::OnLblNewDocLeftClick(wxMouseEvent& evt)
{
    if (!lblNewDoc->IsEnabled())
    {
        evt.Skip();
        return;
    }
    if (cmbFolder->GetValue().IsEmpty())
    {
        MiscTools::MessageBox(_("Missing folder name."), true);
        evt.Skip();
        return;
    }
    if (cmbLabel->GetValue().IsEmpty())
    {
        MiscTools::MessageBox(_("Missing label."), true);
        evt.Skip();
        return;
    }
    if (cmbFormat->GetValue().IsEmpty())
    {
        MiscTools::MessageBox(_("Missing extension."), true);
        evt.Skip();
        return;
    }
    if (!evt.ShiftDown())
    {
        evt.Skip();
        return;
    }
    const wxString emptyString;
    const wxString year = wxVariant(spnYear->GetValue()).GetString();
    const wxString ticks = wxVariant(wxDateTime::Now().GetTicks()).GetString();
    const int counterLength = (int) m_config->Read(_T("/Scanner/Counter/Length"), 2);
    const wxString counter = wxString('0', counterLength);
    const wxString fullDir = dpkRepository->GetPath() + _SEP_
            + year + _SEP_
            + cmbFolder->GetValue();
    const wxString hint = cmbHint->GetValue().IsEmpty()
            ? emptyString
            : cmbHint->GetValue();
    const wxString fileName = cmbLabel->GetValue() + _DASH_
            + ticks + _DASH_
            + hint + (hint.IsEmpty() ? emptyString : _DASH_)
            + counter + _DOT_
            + cmbFormat->GetValue();
    const wxString fullPath = fullDir + _SEP_ + fileName;
    if (wxFileName::FileExists(fullPath))
    {
        MiscTools::MessageBox(fullPath + _(" already present.\n\nAbort."));
        evt.Skip();
        return;
    }
    if (!wxFileName::DirExists(fullDir))
    {
        if (wxDir::Make(fullDir, wxS_DIR_DEFAULT, wxPATH_MKDIR_FULL))
        {
            MiscTools::MessageBox(_("Created : ") + fullDir, true);
        }
        else
        {
            MiscTools::MessageBox(_("Failed to create : ") + fullDir, true);
            txtNewDoc->Clear();
            evt.Skip();
            return;
        }
    }
    txtNewDoc->SetValue(fullPath);
    evt.Skip();
}

void XCB7::OnBtnAbout(wxCommandEvent& evt)
{
    const wxString msg = _APPNAME_ + _SPACE_ + _DASH_ + _SPACE_
            + _APPTITLE_ + _SPACE_ + _DASH_ + _SPACE_
            + _("version") + _SPACE_ + _APPVERSION_ + _NL_ + _NL_
            + _("Copyright : Saleem Edah-Tally, nmset@yandex.com") + _NL_ + _NL_
            + _("License : GPL for the application; GPL/LGPL per file header.");
    MiscTools::MessageBox(msg);
    evt.Skip();
}

void XCB7::OnCloseWindow(wxCloseEvent& evt)
{
    if (m_xsaneWorker->IsScanning())
    {
        MiscTools::MessageBox(_("Abort. File copy in progress."), true);
        evt.Veto();
        return;
    }
    evt.Skip();
}

void XCB7::ForwardFileCopyInfo(const wxString& msg)
{
    txtResults->AppendText(msg);
}

void XCB7::OnTxtNewDocKeyPressed(wxKeyEvent& evt)
{
    if (evt.GetKeyCode() == WXK_BACK)
    {
        txtNewDoc->Clear();
    }
    evt.Skip();
}

void XCB7::OnTxtResultsKeyPressed(wxKeyEvent& evt)
{
    if (evt.GetKeyCode() == WXK_BACK)
    {
        if (evt.ControlDown())
        {
            txtResults->Show(false);
            szMain->Layout();
        }
        else
        {
            txtResults->Clear();
        }
    }
    evt.Skip();
}

void XCB7::OnBtnRefreshScanners(wxCommandEvent& evt)
{
    if (m_xsaneWorker)
        m_xsaneWorker->PopulateScannerList();
    evt.Skip();
}

void XCB7::OnDpkDoubleClick(wxMouseEvent& evt)
{
    const wxString command = _T("xdg-open ") + dpkRepository->GetPath();
    if (wxExecute(command) == 0)
    {
        cout << _("Could not launch default file manager") << endl;
    }
}

///////////////////////////////////////////////////////////////////////////////

TransientPopupEVH::TransientPopupEVH(XCB7* owner, wxComboBox * target,
                                     const wxString& path, const wxString& TTTPath)
{
    m_owner = owner;
    m_target = target;
    m_path = path;
    m_TTTPath = TTTPath;
    m_configItems = m_owner->m_configItems;
}

TransientPopupEVH::~TransientPopupEVH()
{

}

void TransientPopupEVH::OnShow()
{
    const wxString info = m_configItems->Read(_SLASH_ + m_TTTPath + _SLASH_ + m_target->GetValue());
    ConfigComboPopup * configPopup = m_owner->m_ConfigPopup;
    configPopup->GetTxtInfo()->SetValue(info);
}

void TransientPopupEVH::OnSave()
{
    if (m_target->GetValue().IsEmpty()) return;
    ConfigComboPopup * configPopup = m_owner->m_ConfigPopup;
    const wxString item = m_target->GetValue();
    m_configItems->Write(_SLASH_ + m_path + _SLASH_ + MiscTools::MakeUUID(), item);
    m_configItems->Write(_SLASH_ + m_TTTPath + _SLASH_ + item, configPopup->GetTxtInfo()->GetValue());
    m_configItems->Flush();
    m_target->SetToolTip(configPopup->GetTxtInfo()->GetValue());
    const int idx = m_target->FindString(item, true);
    if (idx == wxNOT_FOUND)
        m_target->Append(item);
    m_target->Select(idx);
    //One special case
    if (m_target == m_owner->cmbFormat)
    {
        m_owner->m_config->Write(_T("/Last/Format"), m_target->GetValue());
        m_owner->m_config->Flush();
    }
}

void TransientPopupEVH::OnDelete()
{
    ConfigComboPopup * configPopup = m_owner->m_ConfigPopup;
    const wxString item = m_target->GetValue();
    MiscTools::ConfigRemoveByValue(m_configItems, _SLASH_ + m_path, item);
    m_configItems->DeleteEntry(_SLASH_ + m_TTTPath + _SLASH_ + item);
    m_configItems->Flush();
    m_target->RemoveSelection();
    m_target->Delete(m_target->FindString(item, true));
    m_target->SetValue(wxEmptyString);
    m_target->SelectNone();
    m_target->SetToolTip(NULL);
    configPopup->GetTxtInfo()->Clear();
    //One special case
    if (m_target == m_owner->cmbFormat)
    {
        m_owner->m_config->DeleteEntry(_T("/Last/Format"));
        m_owner->m_config->Flush();
    }
}

///////////////////////////////////////////////////////////////////////////////

XClientData::XClientData(const wxVariant& data)
: wxClientData()
{
    m_data = data;
}

XClientData::~XClientData()
{

}
