/* 
 * File:   XCB7.h
 * Author: Saleem EDAH-TALLY - nmset@yandex.com
 * License : GPL
 * Copyright Saleem EDAH-TALLY - © 2017
 *
 * Created on 27 février 2017, 22:08
 */

#ifndef XCB7_H
#define XCB7_H

#include <wx/wx.h>
#include <wx/config.h>
#include "Resources/UI/cb7.h"
#include "ConfigComboPopup.h"
#include "XSaneWorker.h"
#include "FileWorker.h"

class TransientPopupEVH;
class XSaneWorker;
class FileWorker;

class XCB7 : public CB7
{
public:
    friend class TransientPopupEVH;
    XCB7(wxWindow* parent, wxWindowID id = SYMBOL_CB7_IDNAME,
         const wxString& caption = SYMBOL_CB7_TITLE, const wxPoint& pos = SYMBOL_CB7_POSITION,
         const wxSize& size = SYMBOL_CB7_SIZE, long style = SYMBOL_CB7_STYLE);
    virtual ~XCB7();

    wxConfig * GetConfig() const
    {
        return m_config;
    }

    /**
     * Manages command line options.
     * @return 
     */
    bool ParseCmdLine();

    /**
     * Starts the application. Called if ParseCmdLine() succeeds.
     */
    void Start();

    /**
     * Prints messages received from FileCopy threads.
     */
    void ForwardFileCopyInfo(const wxString& msg);
private:

    /**
     * To edit wxComboBox items on double click.
     */
    ConfigComboPopup * m_ConfigPopup;
    /**
     * An extruded appendix, for clarity.
     */
    XSaneWorker * m_xsaneWorker;
    /**
     * An extruded appendix, for clarity.
     */
    FileWorker * m_fileWorker;
    /**
     * Main application wxConfig object.
     */
    wxConfig * m_config;
    /**
     * User created items in wxComboBox are stored separately. These may be
     * copied from one host to another by moving a file around.
     */
    wxConfig * m_configItems;

    /*
     * An optional tag for wxConfig files. This allows using different realms
     * by specifying the -c command line option.
     */
    wxString m_configTag;

    /**
     * Shows a transient popup to manage wxComboBox items.
     * @param evt
     */
    void OnCmbFolderDClick(wxMouseEvent& evt);
    /**
     * Sets a tooltip text for the selected item.
     * @param evt
     */
    void OnCmbFolderSelect(wxCommandEvent& evt);
    void OnCmbLabelDClick(wxMouseEvent& evt);
    void OnCmbLabelSelect(wxCommandEvent& evt);
    void OnCmbHintDClick(wxMouseEvent& evt);
    void OnCmbHintSelect(wxCommandEvent& evt);
    void OnCmbFormatDClick(wxMouseEvent& evt);
    /**
     * Remembers also the selected item.
     * @param evt
     */
    void OnCmbFormatSelect(wxCommandEvent& evt);
    /**
     * Remembers the selected year.
     */
    void OnSpnYearChange(wxSpinEvent& evt);
    /**
     * Remembers the selected directory.
     */
    void OnDpkRepositoryChange(wxFileDirPickerEvent& evt);
    /**
     * Manages options for a counter suffix
     * @param evt
     */
    void OnLblNewDocRightClick(wxMouseEvent& evt);
    /**
     * Creates a full path for a destination file name.
     * @param evt
     */
    void OnLblNewDocLeftClick(wxMouseEvent& evt);
    void OnBtnAbout(wxCommandEvent& evt);
    /**
     * Aborts application exit if a file is being copied.
     * @param evt
     */
    void OnCloseWindow(wxCloseEvent& evt);

    /**
     * Clears the control on pressing 'back' key.
     * @param evt
     */
    void OnTxtNewDocKeyPressed(wxKeyEvent& evt);

    /**
     * Clears the control on pressing 'back' key.
     * Hides it with 'ctrl + back'.
     * @param evt
     */
    void OnTxtResultsKeyPressed(wxKeyEvent& evt);
    
    /**
     * Find available scanners.
     * @param evt
     */
    void OnBtnRefreshScanners(wxCommandEvent& evt);
    
    /**
     * Open file manager using xdg-open.
     * @param evt
     */
    void OnDpkDoubleClick(wxMouseEvent& evt);
};

/**
 * Implementation of callbacks for ConfigComboPopup.
 * wxComboBox items are managed by these callbacks.
 */
class TransientPopupEVH : public TransientPopupComboEvent
{
public:
    TransientPopupEVH(XCB7 * owner, wxComboBox * target,
                      const wxString& path, const wxString& TTTPath);
    virtual ~TransientPopupEVH();

private:
    XCB7 * m_owner;
    wxComboBox * m_target;
    /**
     * wxConfig path referencing items in a wxComboBox.
     */
    wxString m_path;
    /**
     * wxConfig path referencing tooltip texts for combo items.
     */
    wxString m_TTTPath;
    /**
     * A separate wxConfig for wxComboBox items only.
     */
    wxConfig * m_configItems;

    /**
     * Creates a transient popup allowing to save or delete a wxComboBox item,
     * and manages a corresponding tooltip text.
     */
    void OnShow() override;
    /**
     * Saves an item and a tooltip text, and selects the item.
     * Can also just update a tooltip text.
     */
    void OnSave() override;
    /**
     * Deletes an item and its tooltip text.
     */
    void OnDelete() override;

};

/**
 * For the available scanner list.
 */
class XClientData : public wxClientData {
public :
    XClientData(const wxVariant& data);
    virtual ~XClientData();
    
    wxVariant GetData() const {return m_data;}
private:
    wxVariant m_data;
};
#endif /* XCB7_H */

